// $Id: ConstAddrRndReqMaker.cc,v 1.22.2.8 1998/11/08 05:33:49 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "IOBuffer.h"
#include "ConstAddrRndReqMaker.h"
#include "XactLog.h"
#include "Gadgets.h"
#include "Rnd.h"
#include "globals.h"

#include <math.h>

int ConstAddrRndReqMaker::genObjId() {
	if (TheOpt.theRndSeed > 0) { // randomly distributed object ids
		static RndGen theRnd(TheOpt.theRndSeed);
		// XXX: hard coded zipf(1) distribution
		return (int) (0.5 + exp(TheWorld.zipfOmega()*theRnd() - TheEMConst));
	} else {
		// unique ids (e.g. for populating the cache)
		static int uniqueueId = TheOpt.theSeqStart;
		return uniqueueId++;
	}
}

inline int toId(int n) { return n > 0 ? n : (1-n); }

int genXactId() {
	static RndGen theRnd(TheClock.time().sec() ^ TheClock.time().usec());
	return toId(theRnd.ltrial());
}

void ConstAddrRndReqMaker::doContent() {
	static int unique_id = TheOpt.theUniqStart;
	IOBuffer &buf = theConn->theWriterBuf;

	const int oid = genObjId();
	theCont.theReqHdr->objId(oid);
	const int xid = genXactId();
	theCont.theReqHdr->xactId(xid);

	// form HTTP request
	// req-line prefix
	buf << "GET http://" << theAddr.host();
	if (theAddr.port() != 80)
		buf << ':' << theAddr.port();

	// url
	if (const char *url = TheWorld.url(oid)) {
		// preloaded urls
		buf << url;
	} else {
		// standard format
		buf << "/_" << oid << '/' << TheWorld.id();

		// signature (XXX: hardcoded!)
		buf << "/polyGraph";

		if (unique_id >= 0)
			buf << "/" << unique_id++;
	}

	// req-line trailer
	//buf << " HTTP/1.1" << crlf; // note: 1.1 Implies persistency
	buf << " HTTP/1.0" << crlf;

	// headers
	buf << "X-Xact: " << xid << crlf;
	buf << "Accept: */*" << crlf;
	// ...

	// add crlf
	buf << crlf;

	// done!
	maxOffsetHere();

	// XXX: kludge, should form a real header, but that would lead to copying/allocating stuff...
	theCont.theReqHdr->hdrSize(sizeTotal());
	sendContChange();
}
