// $Id: PortMgr.cc,v 1.1.2.5 1998/11/18 18:49:40 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include "Socket.h"
#include "PortMgr.h"
#include "Log.h"
#include "globals.h"

PortMgr ThePortMgr;

void PortMgr::configure(int aPortMin, int aPortMax, bool ephPorts) {
	Assert(!theBusyPorts.count());

	thePortMin = aPortMin;
	thePortMax = aPortMax;
	isEphPorts = ephPorts;

	if (isEphPorts) {
		SysLog << "PortMgr uses ephemeral ports" << endl;
	} else {
		const int count = thePortMax - thePortMin + 1;
		theValidPorts.resize(count);
		theVoidPorts.resize(count);
		theBusyPorts.resize(count);

		// push free ports
		for (int p = thePortMin; p <= thePortMax; ++p)
			theValidPorts.enqueue(p);

		SysLog << "PortMgr manages ports " << thePortMin << " thru " << thePortMax << endl;
	}
}

int PortMgr::bind(Socket &s) {
	int port = -1;

	// find idle port
	if (isEphPorts) {
		if (s.bind(0))
			port = s.lport();
		else
			SysLog << "bind to ephemeral port failed" << ende;
	} else {
		port = findPort(theValidPorts, s);
		if (port < 0)
			port = findPort(theVoidPorts, s);

		if (port < 0) {
			SysLog << "no ports left;"
				<< " busy: " << theBusyPorts.count() 
				<< " range: [" << thePortMin << ',' << thePortMax << ']'
				<< ende;
			Assert(theValidPorts.empty() && theVoidPorts.empty());
			Assert(theBusyPorts.count() == thePortMax-thePortMin+1);
		}
	}

	if (port >= 0) // found idle port
		reserve(port);

	return port;
}

void PortMgr::reserve(int port) {
	Assert(port >= 0);
	if (!isEphPorts) {
		Assert(port >= thePortMin && port <= thePortMax);
		Assert(!isBusy(port));
		beBusy(port);
	}
}

void PortMgr::free(int port, bool good) {
	Assert(port >= 0);
	if (!isEphPorts) {
		Assert(port >= thePortMin && port <= thePortMax);
		Assert(isBusy(port));
		if (good)
			theValidPorts.enqueue(port);
		else
			theVoidPorts.enqueue(port);
		beIdle(port);
	}
}


int PortMgr::findPort(Ring<int> &ports, Socket &s) {
	for (int i = ports.count(); i > 0; --i) {
		const int port = ports.dequeue();
		Assert(port >= thePortMin && port <= thePortMax);
		if (isBusy(port))
			continue;
		if (!s.bind(port)) {
			cerr << "PortMgr::findPort failed to bind to " << port << ": (" << errno << ") " << strerror(errno) << endl;
			theVoidPorts.enqueue(port);
		}else
			return port;
	}
	return -1;
}
