// $Id: RepCBlock.cc,v 1.8.2.1 1998/09/22 07:43:36 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include <ctype.h>
#include <stdlib.h>
#include <iomanip.h>

#include "RepCBlock.h"

static const char *TheContMagic = "PoLyGrApH";
static const Size TheContMagicSize = strlen(TheContMagic);
static const int TheCBlockMaxSize = TheContMagicSize + 2*(1+10) + 1;

void RepCBlock::load(IOBuffer &buf) {
	// check that '\0' block terminator is present
	if (!buf.contSize() || (Size)strlen(buf.content()) >= buf.contSize()) {
		if (buf.contSize() > TheCBlockMaxSize)
			buf.errFmt(true);
		return;
	}

	int len, timestamp;
	const Size startOffset = buf.outOffset();
	buf >> TheContMagic >> ws >> len >> ws >> timestamp >> ws >> '\0';
	if (!buf.errFmt()) {
		theContSize = len;
		/* theExpire = timestamp; */
		theBlockSize = buf.outOffset() - startOffset;
	}
}

void RepCBlock::store(IOBuffer &buf) {
	const Size startOffset = buf.inOffset();
	const int timestamp = 12345; // XXX: implement
	buf << TheContMagic << endl << (int)theContSize << endl << timestamp << endl << '\0';
	theBlockSize = buf.inOffset() - startOffset;
}

ostream &RepCBlock::print(ostream &os) const {
	return os << "magic: " << TheContMagic << "cont: " << theContSize << endl;
}
