// $Id: Trace.cc,v 1.1.2.2 1998/09/22 07:43:45 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"
#include "Trace.h"
#include "Gadgets.h"


Trace::Trace(const char *aLabel, const char *aMeasure, int aWinCount):
	theWinCount(aWinCount), theCount(0),
	theVals(new int[aWinCount]),
	theLabel(aLabel), theMeasure(aMeasure) {
}

int Trace::keptCount() const {
	return min(theCount, theWinCount);
}

static
int int_cmp(const void *A, const void *B) {
	return *(const int*)A - *(const int*)B;
}

// record 100 percentiles
void Trace::perc(Array<int> &perc) const {
	const int count = keptCount();
	perc.reset();
	if (count <= 0)
		return;

	// clone and sort
	int *trace = new int[count];
	memcpy(trace, theVals, sizeof(*trace) * count);
	qsort(trace, count, sizeof(*trace), &int_cmp);
	
	perc.stretch(101);
	const double perc_step = count/100.0;

	// record percentiles
	for (double i = 0; perc.count() < 100 && i < count; i += perc_step)
		perc.append(trace[(int)i]);
	perc.append(trace[count-1]); // max

	delete[] trace;
}

// prints stats report for the trace
void Trace::report(ostream &os) const {

	os << "Trace: " << theLabel << " [" << theMeasure << "] {" << endl;
	theStat.report(os);

	// percentiles
	Array<int> ps;
	perc(ps);
	if (ps.count()) {
		Assert(ps.count() == 101);
		os << endl << "\t Perc:";
		for (int p = 0; p <= 100; p += 10)
			os << "\t " << setw(5) << p << '%';
		os << endl << "\t Vals:";
		for (int p = 0; p <= 100; p += 10)
			os << "\t " << setw(6) << ps[p];
		os << endl;
	} else {
		os << endl << "\t Perc: not available" << endl;
	}

	os << '}' << endl;
}
