#ifndef _ARRAY_H_
#define _ARRAY_H_ "$Id: Array.h,v 1.6.2.1 1998/09/22 07:43:56 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <string.h>

// Array is an array of <Item> items with an unlimited capacity

template <class Item>
class Array {
	public:
		Array(int aCapacity = 0): theItems(0), theCapacity(aCapacity), theCount(0) { theItems = new Item[aCapacity]; }
		~Array() { delete[] theItems; }

		bool full() const { return theCount >= theCapacity; }
		int capacity() const { return theCapacity; }
		int count() const { return theCount; }

		Item &item(int idx) { return theItems[idx]; }
		const Item &item(int idx) const { return theItems[idx]; }
		Item &operator [](int idx) { return item(idx); }
		const Item &operator [](int idx) const { return item(idx); }
		inline void append(const Item &i);

		void reset() { theCount = 0; }
		void stretch(int n) { if (n > theCapacity) grow(n); }
		void pop(int n) { if (theCount >= n) theCount -= n; else abort(); }
		inline void shift(int n);

	protected:
		void grow() { grow(theCapacity <= 0 ? 16 : theCapacity*2); }
		inline void grow(int newCap);

	protected:
		Item *theItems;
		int theCapacity;
		int theCount;
};

/* searchable array */
template <class Item>
class SchArray: public Array<Item> {
	public:
		SchArray(int aCapacity = 0): Array<Item>(aCapacity) {}
		inline bool find(Item item, int &idx) const; // first matching item
		inline bool findOther(Item item, int &idx) const; // first different item
};


/* implementaion of in-lined methods */

template <class Item>
inline
void Array<Item>::append(const Item &item) {
    if (theCount >= theCapacity)
		grow();
    theItems[theCount++] = item;
}

template <class Item>
inline
bool SchArray<Item>::find(Item i, int &idx) const {
	for (idx = 0; idx < count(); ++idx)
		if (item(idx) == i)
			return true;
	return false;
}

template <class Item>
inline
bool SchArray<Item>::findOther(Item i, int &idx) const {
	for (idx = 0; idx < count(); ++idx)
		if (item(idx) != i)
			return true;
	return false;
}


// EEE: not very efficient implementation, but keeps Array general
template <class Item>
inline
void Array<Item>::grow(int newCap) {
	Item *old_items = theItems;
	theItems = new Item[newCap];
	for (int i = 0; i < theCount; ++i)
		theItems[i] = old_items[i];
	delete[] old_items;
	theCapacity = newCap;
}

// discards first n items
template <class Item>
inline
void Array<Item>::shift(int n) {
	
	if (theCount >= n) {
		theCount -= n;
		if (theCount)
			memmove(theItems, theItems+n, sizeof(Item)*theCount);
	}
}

/* commonly used arrays */
typedef Array<int> IntArray;

#endif
