// $Id: ConnMutex.h,v 1.4.2.1 1998/09/22 07:44:01 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _CONN_MUTEX_H_
#define _CONN_MUTEX_H_

#include "Mutex.h"
#include "Connection.h"

// Connection mutex
class ConnMx: public Mutex {
	public:
		ConnMx(const Mutex &mx, Connection *aConn): Mutex(mx), theConn(aConn) {}
		ConnMx(Connection *aConn = 0): theConn(aConn) {}

		void conn(Connection *aConn) { theConn = aConn; }

		Connection &operator *() const { return *safePtr(); }
		operator Connection*() const { return unsafePtr(); }
		Connection *operator ->() const { return safePtr(); }

		ConnMx &operator =(int n) { Assert(!n); theConn = 0; theId = -1; return *this; }

	protected:
		Connection *safePtr() const { Assert(safe()); return theConn; }
		Connection *unsafePtr() const { return safe() ? theConn : 0; }
		bool safe() const { return theConn && locked(); }

	protected:
		mutable Connection *theConn;
};
// theConn above is mutable, to avoid duplication of all "pointer" methods
inline ostream &operator <<(ostream &os, const ConnMx &mx) { return os << (Connection*)mx; }

#endif
