#ifndef _GADGETS_H_
#define _GADGETS_H_ "$Id: Gadgets.h,v 1.15.2.3 1998/11/04 22:31:23 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include <sys/types.h>
#include <netinet/in.h>

#include <iostream.h>

// min and max
template <class T> inline T min(T a, T b) { return (a <= b) ? a : b; }
template <class T> inline T max(T a, T b) { return (a >= b) ? a : b; }


// new/delete[] style string duplication
extern char *StrDup(const char *str, int len); // dups exactly len chars and appends '\0'
inline char *StrDup(const char *str) { return str ? StrDup(str, strlen(str)) : 0; }
extern const char *SubStr(const char *str, char del); // static substring till del

// a better atoi, returns def on error
int xatoi(const char *s, int def = 0);

// set default format for a stream and custom precision
void configureStream(ostream &os, int prec);

// returns maximum number of FDs allowed
extern int GetMaxFD();
// attempts to set max number of FDs; returns actual maxFD set or -1 on failure
extern int SetMaxFD(int maxFD);

// safe port range for a user process
extern int GetMinPort();
extern int GetMaxPort();

// network/host byteorder manipulation on a buffer
extern void hton16(char *buf);
extern void ntoh16(char *buf);
extern void hton32(char *buf);
extern void ntoh32(char *buf);
inline void htonTime(char *buf) { hton32(buf); hton32(buf+4); }
inline void ntohTime(char *buf) { ntoh32(buf); ntoh32(buf+4); }

// network/host byteorder manipulation plus stream io
inline void shton16(ostream &os, unsigned short s) { s = htons(s); os.write(&s, sizeof(s)); }
inline void sntoh16(istream &is, unsigned short &s) { is.read(&s, sizeof(s)); s = ntohs(s); }
inline void shton32(ostream &os, unsigned int i)  { i = htonl(i); os.write(&i, sizeof(i)); }
inline void sntoh32(istream &is, unsigned int &i) { is.read(&i, sizeof(i)); i = ntohl(i); }
inline void sntoh32(istream &is, signed int &i) { sntoh32(is, (unsigned int &)i); }

inline void shtonTime(ostream &os, timeval t) { shton32(os, t.tv_sec); shton32(os, t.tv_usec); }
inline void sntohTime(istream &is, timeval &t) { sntoh32(is, (unsigned int &)t.tv_sec); sntoh32(is, (unsigned int &)t.tv_usec); }


extern bool GetConstLine(istream &is, const char *match);


#endif
