// $Id: PortMgr.h,v 1.1.2.3 1998/11/18 18:49:42 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef PORT_MGR_H
#define PORT_MGR_H

#include "BitMask.h"
#include "Ring.h"

class Socket;

// manages allocation of network ports
class PortMgr {
	public:
		PortMgr(): thePortMin(0), thePortMax(0), isEphPorts(false) {}
		~PortMgr() {}

		void configure(int aPortMin, int aPortMax, bool ephPorts);

		int bind(Socket &s); // returns port
		void reserve(int port);
		void free(int port, bool good);

	protected:
		bool isBusy(int port) const { return theBusyPorts.isSet(busyOff(port)); }
		void beBusy(int port) { theBusyPorts.beSet(busyOff(port)); }
		void beIdle(int port) { theBusyPorts.beClear(busyOff(port)); }

		int busyOff(int port) const { return port - thePortMin; }

		int findPort(Ring<int> &ports, Socket &s);

	protected:
		Ring<int> theValidPorts; // never- or successfully-used ports
		Ring<int> theVoidPorts;  // last-use-failed ports
		BitMask theBusyPorts;    // ports currently in use
		int thePortMin;
		int thePortMax;
		bool isEphPorts;
};

// use one global port manager for now
extern PortMgr ThePortMgr;


#endif
