// $Id: Process.h,v 1.12.2.1 1998/09/22 07:44:15 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _PROCESS_H_
#define _PROCESS_H_

#include "AppDef.h"
#include "Thread.h"

#include <iostream.h>

#define MAX_OPT_KID_COUNT 4

// a process is a thread with kids
// note: optimized for "common case" and #kids at most MAX_OPT_KID_COUNT
class Process: public Thread {
	public:
		Process(): theAddKidCount(0), theDelKidCount(0), theLoad(0), isWaitAndDie(false) {}
		virtual ~Process() { if (haveKids()) stopKids(); }

		/* high level optional "tasks" routines */
		bool idle() const { return !theLoad; }
		int load() const { return theLoad; }
		virtual void taskStarted(Thread *) { theLoad++; }
		virtual void taskStopped(Thread *) { if (!--theLoad) beIdle(); }

		/* low level required "kids" routines */
		bool readyToDie() const { return isWaitAndDie && !haveKids(); }
		bool haveKids() const { return theAddKidCount > theDelKidCount; }
		void addKid(int kernIdx) { if (theAddKidCount < MAX_OPT_KID_COUNT) theKids[theAddKidCount] = kernIdx; theAddKidCount++; }
		void delKid(int) { Assert(haveKids()); theDelKidCount++; }

		virtual void checkProgress() {}

	protected:
		void startKid(Thread *th) { th->parent(this); addKid(TheKernel.start(th)); }
		void execKid(Thread *th) { isWaitAndDie = true; startKid(th); }
		void stopKids();

		virtual void beIdle() { beActive(thDone); }

	protected:
		int theKids[MAX_OPT_KID_COUNT]; // kernel Ids
		int theAddKidCount;             // #kids added
		int theDelKidCount;             // #kids deleted
		int theLoad;                    // number of tasks alive
		bool isWaitAndDie;					// true if we should die with the last child
};

#endif
