#ifndef _QUEUE_H_
#define _QUEUE_H_ "$Id: Queue.h,v 1.5.2.1 1998/09/22 07:44:17 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Array.h"

// Queue of <Item>s with an unlimited capacity

template <class Item>
class Queue: public Array<Item> {
	public:
		Queue(int aCapacity = 0): Array<Item>(aCapacity), theOffset(0) {}

		int length() const { return count() - theOffset; }
		int offset() const { return theOffset; }
		bool empty() const { return theOffset >= count(); }

		Item &head() { return item(theOffset); }
		void enqueue(Item item) { append(item); }
		inline Item dequeue();

	public:
		int theOffset;
};

/* implementaion of in-lined methods */

template <class Item>
inline
Item Queue<Item>::dequeue() {
	if (empty())
		abort();
	if (theOffset > theCapacity/2) {
		shift(theOffset);
		theOffset = 0;
	}
	return theItems[theOffset++];
}

#endif
