// $Id: XactLog.h,v 1.6.2.1 1998/09/22 07:44:34 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _XACT_LOG_H_
#define _XACT_LOG_H_

#include "Array.h"
#include "Clock.h"

class ostream;

// expected number of (one direction) IOs per xaction
#define EXP_XIO_COUNT 2

// information collected about a single transaction
class XactRec {
	protected:
		typedef unsigned int TimeOff; // offset in usec

		// IO information
		struct IORec {
			TimeOff theStart;
			TimeOff theEnd;
		};

		// IOs information
		class IOs {
			public:
				IOs(): theCount(0) {}

				int count() const { return theCount; }

				void start(TimeOff off) { curRec().theStart = off; }
				void stop(TimeOff off) { curRec().theEnd = off; theCount++; }

				void save(ostream &) const;    // mem to binary format
				void load(istream &);          // binary format to mem
				void print(ostream &os, int idx) const; // mem to redable stream

			protected:
				IORec &curRec() { return (theCount >= EXP_XIO_COUNT) ? theIOs[EXP_XIO_COUNT] : theIOs[theCount]; }

			protected:
				IORec theIOs[EXP_XIO_COUNT+1];
				unsigned char theCount;
		};

	public:
		XactRec(): theId(-1), theParentId(-1) {}

		Time start() const { return theStart; }
		int id() const { return theId; }

		void id(int anId) { theId = anId; }
		void pid(int aParentId) { theParentId = aParentId; }
		void start(Time aStart) { theStart = aStart; }

		void startRead() { theReads.start(off()); }
		void stopRead() { theReads.stop(off()); }
		void startWrite() { theWrites.start(off()); }
		void stopWrite() { theWrites.stop(off()); }

		void save(ostream &) const;    // mem to binary format
		void load(istream &);          // binary format to mem
		void print(ostream &os, Time globalStart) const; // mem to redable stream
		
	public:
		TimeOff off() const { return (TheClock.now() - theStart).usec(); }

	protected:
		int theId;       // xaction id
		int theParentId; // xaction's parent id

		Time theStart;
		IOs theReads;
		IOs theWrites;
};

// Log for XactRec
class XactLog: public Array<XactRec> {
	protected:
		typedef Array<XactRec> Arr;

	public:
		XactLog();

		void configure(int aStart, int aSize);
		void startNow(); // reconfigure with new start

		bool started() const { return theDebt < theSize; }
		int count() const { return theCount; }

		// returns next record or null
		XactRec *getRec();
		void setId(XactRec *xr) { Assert(xr->id() < 0); xr->id(++theLastId); }

		void save(ostream &) const; // mem to binary format
		void load(istream &);       // binary format to mem

		// binary format redable stream, false on parsing error
		bool print(istream &is, ostream &os);

	protected:
		void saveHeader(ostream &) const;
		int loadHeader(istream &);
		void printHeader(ostream &, int count) const;

	protected:
		int theStartCount; // when to start logging (#transactions)
		int theSize;   // how many xactions to log
		int theDebt;   // #transactions left to log
		int theCount;  // #transactions seen
		int theLastId; // for generating unique ids
};


extern XactLog TheXactLog;

#endif
