#ifndef _GLOBALS_H_
#define _GLOBALS_H_ "$Id: globals.h,v 1.3.2.9 1998/11/17 20:55:00 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Time.h"
#include "Size.h"
#include "Address.h"
#include "Array.h"

// Euler-Mascheroni constant
#define TheEMConst 0.57721566490153286060651209

class World {
	public:
		World(): theId(-1), theCapacity(0), theSize(0), theGoal(0), theComplCount(0), theZipfOmega(-1) {}

		int id() const { return theId; }
		int capacity() const { return theCapacity; }
		//int size() const { return theSize; }
		Size objSizeMean() const { return theObjSizeMean; }
		double zipfOmega() const { return theZipfOmega; }
		const char *url(int oid) const { return (oid > 0 && oid <= theUrls.count()) ? theUrls[oid-1] : 0; }
		int goal() const { return theGoal; }
		bool goalReached() const { return theComplCount >= theGoal; }

		void id(int anId) { theId = anId; }
		void capacity(int aCap) { theCapacity = aCap; syncZipfOmega(); }
		//void size(int aSize) { theSize = aSize; }
		void objSizeMean(Size anObjSize) { theObjSizeMean = anObjSize; }
		void goal(int aGoal) { theGoal = aGoal; }

		void noteCompletion() { theComplCount++; }

		bool loadUrls(const char *fname);

	protected:
		void syncZipfOmega();

	protected:
		int theId;           // world id (to generate unique worlds)
		int theCapacity;     // max number of objects in this world
		int theSize;         // number of objects to simulate
		int theGoal;         // total number of completions to wait for
		Size theObjSizeMean; // mean object size excluding headers
		Array<char*> theUrls;// pre-loaded urls if any

		int theComplCount;   // tasks completed towards the goal
		double theZipfOmega; // harmonic sum derived from capacity
};

// configuration options
class Opt {
	public:
		Opt(); // defaults
		~Opt();

		bool parse(int argc, char *argv[]);
		void report(ostream &os) const;

	protected:
		bool optEqual(const char *name, const char *canonic);
		bool splitVal(const char *value);

	public:
		Address theOriginAddr;
		Address theProxyAddr;
		int theWorldId;
		int theWorldCapacity;
		int theSeqStart;
		int theRndSeed;
		int theUniqStart;
		Size theObjSizeMean;
		Time theConnEstTimout;
		bool useEphPorts;

		int theClientCount;
		int theReqPerClient;
		int theReqTotal;
		Time theLaunchWindow;
		Time theClientThinkTime;
		char *theUrlsFileName;

		int theServerCount;
		Array<int> theServerPorts;
		Time theSrvWriteDelay;
		Time theIdleTimeout;
		bool isRepCachable;

	protected:
		char *theValPref;
		char *theValSuff;
};

extern World TheWorld;
extern Opt TheOpt;
extern int TheMaxFD;            // max #fd we are allowed to use
extern const char *ThePrgName;  // program name

#endif
